% Simulations for alternating phase stimuli
% Matlab work directory must be userProgramSRB

clear;

% Time linear-spacing
minPitch=	30; maxPitch=	10000; numPitches=50;
maxLag=1/minPitch; minLag=1/maxPitch;
lags= linspace(minLag, maxLag, numPitches);
lag_num= length(lags);

% Frequency log-spacing
a= log10(30);
b= log10(10000);
vector= logspace(a, b, numPitches);
lags= 1./vector;

CFlags1=0.5;
CFlags2=15;

% CFlags1=0;
% CFlags2=350;


type_model= 'SACF'; % 'SACF'

plots=0;

ANrepr= 'probability'; % spikes
% ANrepr= 'spikes';

spectrum= 'Low'; % either 'Low' or 'High'

currentF= pwd;
currentF=currentF(1:length(currentF)-15);
currentF=strcat(currentF,'wavFileStore');

currentFolderRef = fullfile(currentF, 'Alternating', 'Ref');
currentFolderAlt = fullfile(currentF, 'Alternating', 'Alt');

folderRef= fullfile(currentFolderRef, spectrum);
folderAlt= fullfile(currentFolderAlt, spectrum);

% Gets all files in directory
listingRef = dir(folderRef);
listingAlt = dir(folderAlt);

namesAlt = struct2cell(listingAlt);
namesRef = struct2cell(listingRef);
namesAlt= namesAlt(1,3:size(namesAlt,2));
namesRef= namesRef(1,3:size(namesRef,2));

num_ref= length(namesRef);
num_Alt= length(namesAlt);

ref_SACFs= zeros(num_ref, lag_num);

for i=1:num_ref, % for each ref stimuli, obtains SACF function
    filename= fullfile(folderRef, namesRef{i});
    [LP_SACF, dt, lags, SACF, ANprob, inputSignal, fs, BFList]= FullPitchModel(filename, lags, CFlags1, CFlags2, ANrepr);
%     LP_SACFmean= mean(LP_SACF,2);
    SACFmean= mean(SACF,2);
    ref_SACFs(i,:)=SACFmean;
    
end

D2value= zeros(num_Alt,1);
bestPitch= zeros(num_Alt,1);

for i=1:num_Alt, % for each mistuning percentage, determines best pitch match
    filename= fullfile(folderAlt, namesAlt{i});
    [LP_SACF, dt, lags, SACF, ANprob, inputSignal, fs, BFList]= FullPitchModel(filename, lags, CFlags1, CFlags2, ANrepr);
%     LP_SACFmean= mean(LP_SACF,2);
    SACFmean= mean(SACF,2)';
    D2=zeros(num_ref,1);
    for j=1:num_ref, % Computes Euclidean distance with all reference stimuli
        D2(j)= Euclidean(SACFmean,ref_SACFs(j,:));
    end
    [D2value(i), index]= min(D2); % Computes minimum Eucl. Dist.
    % Determines pitch from filename
    nameRef=namesRef{index};
    startp=strfind(nameRef,'(');
    endp=strfind(nameRef,')');
    pitch= nameRef(startp+1:endp-3);
    bestPitch(i)=str2double(pitch);
    
%     subplot(211)
%     plot(lags,ref_SACFs(index,:));
%     xlabel('Lag(s)')
%     ylabel('SACF Sine phase')
%     subplot(212)
%     plot(lags,SACFmean);
%     xlabel('Lag(s)')
%     ylabel('SACF ALT phase')
end

bestPitch


