% Simulations for Pitch shift of equally spaced components


clear;

spacing= 'linear';

% Time linear-spacing
minPitch=	30; maxPitch=	10000; numPitches=250;
maxLag=1/minPitch; minLag=1/maxPitch;
lags= linspace(minLag, maxLag, numPitches);
lag_num= length(lags);

% Frequency log-spacing
if strcmp(spacing, 'log')
    a= log10(minPitch);
    b= log10(maxPitch);
    vector= logspace(a, b, numPitches);
    lags= 1./vector;
end

CFlags1=0.5;
CFlags2=15;

% CFlags1=0;
% CFlags2=350;

type_model= 'SACF'; % 'SACF'

plots=0;

ANrepr= 'probability'; % spikes
% ANrepr= 'spikes';

currentF= pwd;
currentF=currentF(1:length(currentF)-15);
currentFWav=strcat(currentF,'wavFileStore');
folderRef = fullfile(currentFWav, 'Shifted', 'Ref');
folderM = fullfile(currentFWav, 'Shifted', 'S');

outputFolder= fullfile(currentF, 'Results', 'Shifted');
if ~isdir(outputFolder),
    mkdir(outputFolder);
end

% Gets all files in directory
listingRef = dir(folderRef);
listingM = dir(folderM);

namesM = struct2cell(listingM);
namesRef = struct2cell(listingRef);
namesM= namesM(1,3:length(namesM));
namesRef= namesRef(1,3:length(namesRef));

num_ref= length(namesRef);
num_M= length(namesM);

ref_SACFs= zeros(num_ref, lag_num);

% for i=1:num_ref, % for each ref stimuli, obtains SACF function
%     filename= fullfile(folderRef, namesRef{i});
%     [LP_SACF, dt, lags, SACF, ANprob, inputSignal, fs, BFList]= FullPitchModel(filename, lags, CFlags1, CFlags2, ANrepr);
% %     LP_SACFmean= mean(LP_SACF,2);
%     SACFmean= mean(SACF,2);
%     ref_SACFs(i,:)=SACFmean;
%     
% end

f0=100;

D2value= zeros(num_M,1);
bestPitch= zeros(num_M,1);
Harmnum= zeros(num_M,1);

for i=1:num_M, % for each mistuning percentage, determines best pitch match
    shortname= namesM{i};
    filename= fullfile(folderM, shortname);
    [LP_SACF, dt, lags, SACF, ANprob, inputSignal, fs, BFList]= FullPitchModel(filename, lags, CFlags1, CFlags2, ANrepr);
%     LP_SACFmean= mean(LP_SACF,2);
    SACFmean= mean(SACF,2)';
    SACFmean2= SACFmean(lags<11/(10*f0));
    lags2=lags(lags<11/(10*f0));
    SACFmean3= SACFmean2(lags2>9/(10*f0));
    lags3=lags2(lags2>9/(10*f0));
    [maxLag, index]= max(SACFmean3);
%     plot(lags,SACFmean)
    bestPitch(i)= 1/lags3(index);
    shortlen= length(shortname);
    k= strfind(shortname, 'Hn=');
    k2=strfind(shortname, 'shift=');
    Hn= str2double(shortname(k+3:k2-1));
    shift=str2double(shortname(k2+6:shortlen-4));
    Hnshift=Hn+shift/100;
    Harmnum(i)=Hnshift;
%     
%     D2=zeros(num_ref,1);
%     for j=1:num_ref, % Computes Euclidean distance with all reference stimuli
%         D2(j)= Euclidean(SACFmean,ref_SACFs(j,:));
%     end
%     [D2value(i), index]= min(D2); % Computes minimum Eucl. Dist.
%     % Determines pitch from filename
%     nameRef=namesRef{index};
%     k3=strfind(nameRef,'(');
%     k4=strfind(nameRef,'Hz');
%     pitch= nameRef(k3+1:k4-1);
%     bestPitch(i)=str2double(pitch);
end

[M, I]= sort(Harmnum);
bestPitch= bestPitch(I);

nombreFig= fullfile(outputFolder, 'Shifted');
nombreFig= strcat(nombreFig,'#', num2str(numPitches), spacing, num2str(floor(10*CFlags1)), '-', num2str(floor(10*CFlags2)));
h=figure('Name','Shifts','visible', 'off');
plot(M,bestPitch, '+');
xlabel('Harmonic number of lowest component')
ylabel('Pitch(Hz)')
grid on
saveas ( h, nombreFig, 'png' )
saveas ( h, nombreFig, 'fig' )
close(h)
